@@ -22,6 +22,10 @@ class ShopBase(models.Model):
     def shopify(self):
         return shopify
 
+    @property
+    def shopify_app_api_secret(self):
+        return apps.get_app_config("shopify_app").SHOPIFY_API_SECRET
+
     @property
     def shopify_session(self):
         api_version = apps.get_app_config("shopify_app").SHOPIFY_API_VERSION
