@@ -1,3 +1,6 @@
+import base64
+import hashlib
+import hmac
 from django.views import View
 from django.urls import reverse
 from django.conf import settings
@@ -6,10 +9,12 @@ from django.views.decorators.clickjacking import xframe_options_exempt
 from django.utils.decorators import method_decorator
 
 from rest_framework.views import APIView
+from rest_framework.response import Response
+from rest_framework.exceptions import APIException
 
 import shopify
 
-from shopify_app.utils import get_shop_model
+from shopify_app.utils import get_function_from_string, get_shop_model
 from shopify_app.decorators import shopify_embed
 
 
@@ -87,3 +92,47 @@ class EndTokenRequestView(View):
         query = request.META['QUERY_STRING']
 
         return redirect(f"{host}{path}?{query}")
+
+
+class WebhookAuthenticationFailed(APIException):
+    status_code = 401
+    default_detail = 'authentication failed'
+
+
+class WebhookMixin:
+
+    def webhook_request_is_valid(self, secret):
+        hmac_header = self.request.META.get('HTTP_X_SHOPIFY_HMAC_SHA256')
+        if not hmac_header:
+            raise WebhookAuthenticationFailed()
+
+        secret = secret.encode('utf-8')
+        digest = hmac.new(secret, self.request.body, hashlib.sha256).digest()
+        computed_hmac = base64.b64encode(digest)
+        received_hmac = hmac_header.encode('utf-8')
+        is_valid = hmac.compare_digest(computed_hmac, received_hmac)
+        return is_valid
+
+
+class MandatoryWebhooksView(WebhookMixin, APIView):
+
+    topic = None
+
+    def post(self, request, *args, **kwargs):
+
+        try:
+            domain = request.META['HTTP_X_SHOPIFY_SHOP_DOMAIN']
+            Shop = get_shop_model()
+            shop = Shop.objects.get(shopify_domain=domain)
+        except (KeyError, Shop.DoesNotExist):
+            return Response(status=404)
+
+        if not self.webhook_request_is_valid(shop.shopify_app_api_secret):
+            raise WebhookAuthenticationFailed()
+
+        func = get_function_from_string(
+            settings.SHOPIFY_GDPR_WEBHOOK_CALLBACK
+        )
+        func(self.topic, request.data)
+
+        return Response(status=200)
